
from ast import Dict
from typing import Any
from gisweb_ads.schema import IDocumento, IConfig, ILoginRet, IResult, IProtocollo
from gisweb_ads.soap_attachments import with_soap_attachment
from suds.client import Client
import logging
import os, base64, uuid
import json
from datetime import datetime
from pydantic import BaseModel
from jinja2 import Environment, PackageLoader, select_autoescape
import time

_logger = logging.getLogger('gisweb-ads')


env = Environment(
    loader=PackageLoader("gisweb_ads"),
    autoescape=select_autoescape()
)

class Protocollo:
    
    config:IConfig
    data:IProtocollo
    wsClient:Any
    DST:str | None
    wsError: IResult
  
    def __init__(self, config:IConfig, data:IProtocollo) -> None:
        self.data = data
        self.config = config
        self.DST = ''
        if self.config.FAKE:
            return
        
        try:
            self.wsClient = Client(config.wsUrl)
            result:ILoginRet = self.wsClient.service.login(config.wsEnte, config.wsUser, config.wsPassword)
            if result.lngErrNumber==0:
                self.DST=result.strDST
            else:
                self.wsError = IResult(strErrString=result.strErrString,lngErrNumber=result.lngErrNumber)

        except Exception as error:
            self.wsError = IResult(strErrString="ERRORE NELLA CONNESSIONE A WS",lngErrNumber=999)


    def __inserisciDocumento(self, documento:IDocumento) -> IDocumento | None:
        
        fName = documento.nome.encode('ascii','ignore')
        mime = documento.mimetype.encode('ascii','ignore')
        content = documento.content.read()
        result = with_soap_attachment(self.wsClient.service.inserimento, [content,fName,mime], self.config.wsUser, self.DST)

        try:
            fName = documento.nome.encode('ascii','ignore')
            mime = documento.mimetype.encode('ascii','ignore')
            content = documento.content.read()
            result = with_soap_attachment(self.wsClient.service.inserimento, [content,fName,mime], self.config.wsUser, self.DST)
        except Exception as error:
            _logger.info(error)
            return 

        if result.lngErrNumber != 0:
            _logger.info(result.strErrString)
            return
        
        dd = documento.dict()
        dd["id"] = result.lngDocID
        return IDocumento(**dd)
        

    def __getSegnatura(self):
        
        #inserisco allegato principale
        newDoc = self.__inserisciDocumento(self.data.Principale)
        if newDoc:
            self.data.Principale = newDoc
            
        #inserisco altri allegati
        allegati = []
        for documento in self.data.Allegati:
            newDoc = self.__inserisciDocumento(documento)
            if newDoc:
                allegati.append(newDoc)

        self.data.Allegati = allegati
            
        template = env.get_template("segnatura_e.xml")
        return template.render(self.data)


    def protocollaDocumento(self, messageData=None, testXml=False) -> IResult:
        
        if self.config.FAKE:
            time.sleep(30)
            return IResult(lngNumPG=444,lngAnnoPG=2023, strDataPG='01/01/2222') 
        
     
        if not self.DST:
            return self.wsError
        

        if messageData:
            
            principale=messageData.get("principale")
            allegati=messageData.get("allegati")
            oggetto=messageData.get("protocollo_oggetto")
            destinatari_a=messageData.get('destinatari_a',[])
            destinatari_aoo=messageData.get('destinatari_aoo',[])
            destinatari_cc=messageData.get('destinatari_cc',[])
            flusso='U'

            # se il flusso è in uscita i destinatari sono obbligatori
            if not destinatari_a:
                message = u"Mancano i destinatari per la protocollazione"
                return IResult(lngErrNumber=999, strErrString="Mancano i destinatari per la protocollazione")
            
            ## ci vanno anche quelli in cc?
            self.Destinatari = destinatari_a + destinatari_cc
            self.DestinatariAOO = destinatari_aoo
            self.DestinatariA=destinatari_a
            self.DestinatariCC=destinatari_cc

        else:
            ### prot in entrata
            pippo=1
            

        xmlSegnatura = self.__getSegnatura()
        
        if testXml:       
            return IResult(strErrString=xmlSegnatura)

        #Protocollazione Documento
        try:
            result = with_soap_attachment(self.wsClient.service.protocollazione,
                                          [xmlSegnatura, 'profilazione', 'text/xml'], self.config.wsUser, self.DST)  
            return result
            
        except Exception as e:
            return IResult(lngErrNumber=999, strErrString= str(e))
        



        # se protocollazione in uscita aggiungo la segnatura e oggetto
        ''''
        if flusso=='U':
            self.numero_protocollo="%07d" %result['lngNumPG']
            self.data_protocollo=datetime.today().strftime("%d/%m/%Y")
            # Segnatura di protocollazione
            tplSegnatura = ViewPageTemplateFile("templates/segnatura_pec.pt")
            xmlSegnatura = tplSegnatura(self)
        '''




# -*- coding: utf-8 -*-
import uuid
import logging
import xml.etree.ElementTree as ET
from gisweb_ads.schema import IResult

_logger = logging.getLogger('gisweb.sicraweb')

def parseMessage(xmlMessage) -> IResult:

  root = ET.fromstring(xmlMessage)
  ret = IResult(lngErrNumber=666, strErrString='Errore nel parser del risultato')
  if len(root[0])==2:
    result = root[0][1]
  else:
    result = root[0][0][0]

  for node in result:
    if 'lngDocID' in node.tag:
      ret.lngDocID = int(node.text or '0')
    if 'lngErrNumber' in node.tag:
      ret.lngErrNumber = int(node.text or '0')
    if 'strErrString' in node.tag:
      ret.strErrString = node.text or ''
    if 'lngNumPG' in node.tag:
      ret.lngNumPG = int(node.text or '0')
    if 'lngAnnoPG' in node.tag:
      ret.lngAnnoPG = int(node.text or '0')
    if 'strDataPG' in node.tag:
      ret.strDataPG = node.text or ''
      
    if ret.lngErrNumber == 0:
      ret.strErrString = ''
      
  return ret



def with_soap_attachment(suds_method, attachment_data, *args, **kwargs) -> IResult:
    """ Add an attachment to a suds soap request.

    attachment_data is assumed to contain a list:
      ( <attachment content>, <content id>, <mime-type> )

    The attachment content is only required required list element.

    http://stackoverflow.com/questions/17046779/passing-file-attachments-in-soap-using-suds

    """

    from suds.transport import Request

    # Suds doesn't currently support SOAP Attachments, so we have to build our
    # own attachment support, using parts of the suds library

    MIME_DEFAULT = 'text/plain'
    attachment_encoding = '8bit'
    attachment_mimetype = MIME_DEFAULT
    attachment_id = ''
    soap_method = suds_method.method
    data = ''
    
    
    if len(attachment_data) == 3:
        data, attachment_id, attachment_mimetype = attachment_data
    elif len(attachment_data) == 2:
        data, attachment_id = attachment_data
    elif len(attachment_data) == 1:
        data = attachment_data
        attachment_id = uuid.uuid4()

    # Generate SOAP XML appropriate for this request
    binding = soap_method.binding.input
    soap_xml = binding.get_message(soap_method, args, kwargs)

    # Prepare MIME headers & boundaries
    boundary_id = 'uuid:%s' % uuid.uuid4()
    root_part_id ='uuid:%s' % uuid.uuid4()
    request_headers = {
      'Content-Type': '; '.join([
          'multipart/related',
          'type="text/xml"',
          'start="<%s>"' % root_part_id,
          'boundary="%s"' % boundary_id,
        ]),
      'SOAPAction':''
    }
    soap_headers = '\n'.join([
      'Content-Type: text/xml; charset=UTF-8',
      'Content-Transfer-Encoding: 8bit',
      'Content-Id: <%s>' % root_part_id,
      '',
    ])
    attachment_headers = '\n'.join([
      'Content-Type: %s' % attachment_mimetype,
      'Content-Transfer-Encoding: %s' % attachment_encoding,
      'Content-Id: <%s>' % attachment_id,
      '',
    ])

    # Build the full request
    request_text = '\r\n'.join([
      '',
      '--%s' % boundary_id,
      soap_headers,
      str(soap_xml),
      '--%s' % boundary_id,
      attachment_headers,
      str(data),
      '--%s--' % boundary_id
    ])

    # Stuff everything into a request object
    headers = suds_method.client.options.headers.copy()
    headers.update(request_headers)
    request = Request(suds_method.client.wsdl.url, request_text.encode('utf-8'))
    request.headers = headers
    # Send the request
    #_logger.info(request_text)

    response = suds_method.client.options.transport.send(request)
    if response.code==200:
      return parseMessage(response.message)
    else:
      return IResult(lngErrNumber=999, strErrString= "ERRORE NELLA RISPOSTA DEL SERVIZIO")




from typing import Any, List, Dict
from pydantic import BaseModel, EmailStr, HttpUrl, PrivateAttr
from enum import Enum

class IAzioneEnum(str, Enum):
    ESEGUI = "ESEGUI"
    CARICO = "CARICO"
    
class ISmistamentoEnum(str, Enum):
    CONOSCENZA = "CONOSCENZA"
    COMPETENZA = "COMPETENZA"

    
class IConfig(BaseModel):
    wsUrl: str
    wsUser: str
    wsEnte: str
    wsPassword: str
    FAKE: bool = False
    
class BaseRet(BaseModel):
    lngErrNumber: int = 0
    strErrString: str = ''

class ILoginRet(BaseRet):
    strDST: str | None
    
class IResult(BaseRet):
    lngNumPG: int = 0
    lngAnnoPG: int = 0
    strDataPG: str = ''
    lngDocID: int = 0

    
class IPersona(BaseModel):
    IndirizzoTelematico: str
    Denominazione: str | None
    Nome: str
    Cognome: str
    TitoloDitta: str | None
    CodiceFiscaleDitta: str | None
    IndirizzoTelematicoDitta: str | None
    TipoMittente: str | None
    CodiceFiscale: str
    Titolo: str

class IDocumento(BaseModel):
    id: int | None
    descrizione: str
    tipo: str
    nome: str
    content: Any
    size: int
    mimetype: str
    ext: str


class IUser(BaseModel):
    username: str
    password: str

class IAmministrazione(BaseModel):
    Denominazione: str
    CodiceAOO: str
    CodiceEnte: str
    IndirizzoTelematico: EmailStr

class IFascicolo(BaseModel):
    numero: str
    anno:str
    
class IParametro(BaseModel):
    nome: str
    valore: str

class IProtocollo(BaseModel):
    Amministrazione: IAmministrazione
    Mittente: List[IPersona]
    Flusso: str = 'E'
    Oggetto: str
    Titolario: str
    UO: str  
    Fascicolo: IFascicolo
    Principale: IDocumento
    Allegati: List[IDocumento]
    NumeroRegistrazione: str = '0'
    DataRegistrazione: str = '0'
    Applicativo: str = 'AGSPR'
    Parametri: List[IParametro]


class IMessageData(BaseModel):
    pass


